use log::info;
use owo_colors::OwoColorize;
use tide::{Middleware, Next, Request, Result};

#[derive(Default)]
pub struct LoggerMiddleware {}

impl LoggerMiddleware {
    pub fn new() -> Self {
        Self {}
    }
}
#[tide::utils::async_trait]
impl<State: Clone + Send + Sync + 'static> Middleware<State> for LoggerMiddleware {
    async fn handle(&self, req: Request<State>, next: Next<'_, State>) -> Result {
        let logstr = format!("{} {}", req.method().green(), req.url().path());

        let res = next.run(req).await;
        info!("{} <-> {}", logstr, res.status());
        Ok(res)
    }
}

#[derive(Default)]
pub struct MinifyMiddleware {
    pub minify_cfg: minify_html::Cfg,
}

impl MinifyMiddleware {
    pub fn new() -> Self {
        let mut cfg = minify_html::Cfg::new();
        cfg.keep_comments = true;
        Self { minify_cfg: cfg }
    }
}
#[tide::utils::async_trait]
impl<State: Clone + Send + Sync + 'static> Middleware<State> for MinifyMiddleware {
    async fn handle(&self, req: Request<State>, next: Next<'_, State>) -> Result {
        let mut res = next.run(req).await;
        match res.content_type() {
            Some(mime) => match mime.subtype() {
                "html" | "css" => {
                    let body = res.take_body().into_bytes().await?;
                    let new_body = minify_html::minify(&body, &self.minify_cfg);
                    res.set_body(new_body);
                }
                _ => {}
            },
            None => {}
        };
        Ok(res)
    }
}
